"""Request data steps."""
from behave import given


@given("request data is available")
def request_data_is_available(context):
    """Make sure request data is available."""
    if not hasattr(context, "request_data"):
        context.request_data = {}


@given('the request has "{key}" with "{value}"')
def request_has_key_with_value(context, key, value):
    """Add a key with a value to the request data."""
    context.execute_steps("Given request data is available")
    context.request_data[key] = value


@given('the request has "{key}" boolean {value}')
def request_has_key_with_boolean_value(context, key, value):
    """Add a key with a boolean value to the request data."""
    context.execute_steps("Given request data is available")
    value = value.lower()
    if value == "true":
        value = True
    elif value == "false":
        value = False
    else:
        raise ValueError(f"Invalid boolean value: {value}")
    context.request_data[key] = value


@given('the request has "{key}" with {value}')
def request_has_key_with_numeric_value(context, key, value):
    """Add a key with a value to the request data."""
    context.execute_steps("Given request data is available")
    if "." in value:
        value = float(value)
    else:
        value = int(value)
    context.request_data[key] = value


@given('the request has values in "{key}"')
def request_has_values_in_key(context, key):
    """Add a key with a value to the request data."""
    context.execute_steps("Given request data is available")
    context.request_data[key] = context.table.rows



"""Model create steps."""
from behave import given


@given('a "{model_name}" with values')
def create_model_with_fields(context, model_name):
    """Create a model with the given values."""
    context.execute_steps(f'Given a "{model_name}" model is available')
    model = context.models[model_name]
    context.test.assertTrue(context.table)
    context.test.assertTrue(context.table[0].as_dict())
    setattr(context, model_name, model.objects.create(**context.table[0].as_dict()))
    context.test.assertTrue(getattr(context, model_name))



"""Model filter steps."""
from behave import then


@then('a "{model_name}" with values exists')
def model_is_available_with_field_value(context, model_name):
    """Check if a model with the given values exists."""
    context.execute_steps(f'Given a "{model_name}" model is available')
    model = context.models[model_name]
    context.test.assertTrue(context.table)
    context.test.assertTrue(context.table[0].as_dict())
    setattr(
        context, model_name, model.objects.filter(**context.table[0].as_dict()).first()
    )
    context.test.assertTrue(getattr(context, model_name))


@then('a "{model_name}" with values does not exist')
def model_is_not_available_with_field_value(context, model_name):
    """Check if a model with the given values does not exist."""
    context.execute_steps(f'Given a "{model_name}" model is available')
    model = context.models[model_name]
    setattr(
        context, model_name, model.objects.filter(**context.table[0].as_dict()).first()
    )
    context.test.assertTrue(getattr(context, model_name) is None)



"""Response status steps."""
import json

from behave import then
from django.http import JsonResponse


@then("a response is available")
def response_is_available(context):
    """Check that a response is available."""
    context.test.assertTrue(getattr(context, "response", None) is not None)


@then('status code "{status_code}" is returned')
def response_status_code_is(context, status_code):
    """Check that the response status code is the given value."""
    context.execute_steps("Then a response is available")
    print(context.response)
    if hasattr(context.response, "data"):
        if not isinstance(context.response, JsonResponse):
            context.response.data = json.loads(context.response.content)
        print(context.response.data)
        print(context.response.content)
    context.test.assertEqual(context.response.status_code, int(status_code))



"""Model existence steps."""
import django.apps
from behave import given, step


@given("models are available")
def models_are_available(context):
    """Load all models."""
    models = django.apps.apps.get_models(
        include_auto_created=True, include_swapped=True
    )
    # pylint: disable=protected-access
    context.models = {
        **{model._meta.model_name: model for model in models},
        **{model._meta.object_name: model for model in models},
        **{model._meta.db_table: model for model in models},
        **{model._meta.verbose_name: model for model in models},
        **{model._meta.verbose_name_plural: model for model in models},
    }


@step('a "{model_name}" model is available')
def model_is_available(context, model_name):
    """Check if a model is available."""
    if not getattr(context, "models", None):
        context.execute_steps("Given models are available")
    model = context.models.get(model_name)
    context.test.assertTrue(model)
